/*
  Programmme pour mettre le circuit blanc en mode automatique et pour enregistrer les données des capteurs du shield MKR ENV.
*/

#include "RTClib.h"  // bibliothèque horloge rtc
#include "ds3231.h"
#include <Wire.h>  // bibliothèque communication wire
#include <Arduino_MKRENV.h>
#include <SD.h>  // bibliothèque écriture carte SD
#include <SPI.h>  // bibliothèque bus SPI (utilisé pour carte SD)

RTC_DS3231 rtc;  // PCF8523 rtc; ou DS3231 // déclaration RTC

DateTime Dtime;
/* DateTime::DateTime(uint16_t year, uint8_t month, uint8_t day, uint8_t hour,
                   uint8_t min, uint8_t sec) {
}*/

const int SDCARD_CS = 4;  // Défini la carte SD sur le port 4
const int PIN_LED_BUILTIN = 6;
const int PIN_CMD_TRANSISTOR = 7;

uint8_t ui8_sleep_period = 1;  // the sleep interval in minutes between 2 consecutive alarms

void set_next_alarm(void)
{
  struct ts S_t;

  DS3231_get(&S_t);

  // calculate the minute when the next alarm will be triggered
  unsigned char uc_wakeup_min = (S_t.min / ui8_sleep_period + 1) * ui8_sleep_period;
  if (uc_wakeup_min > 59) {
    uc_wakeup_min -= 60;
  }

  // flags define what calendar component to be checked against the current time in order
  // to trigger the alarm
  // A2M2 (minutes) (0 to enable, 1 to disable)
  // A2M3 (hour)    (0 to enable, 1 to disable)
  // A2M4 (day)     (0 to enable, 1 to disable)
  // DY/DT          (dayofweek == 1/dayofmonth == 0)
  uint8_t ui8_flags[4] = { 0, 1, 1, 1 };

  // set Alarm2. only the mn is set since we ignore the hour and day component
  DS3231_set_a2(uc_wakeup_min, 0, 0, ui8_flags);

  // activate Alarm2
  DS3231_set_creg(DS3231_CONTROL_INTCN | DS3231_CONTROL_A2IE);
}


void setup()
{
 //// start serial port at 9600 bps:
  Serial.begin(9600);
////  while (!Serial) {
    ;  // wait for serial port to connect. Needed for native USB port only
////  }
  Serial.println("Start system..."); 

  rtc.begin();                                                                                                           //lance le RTC  
  pinMode(PIN_LED_BUILTIN, OUTPUT);
  digitalWrite(PIN_LED_BUILTIN, LOW);                                                                                                //Sortie pin led                       
  pinMode(PIN_CMD_TRANSISTOR, OUTPUT);                                                                                        //Sortie pin 7 HIGH vers transistor
  digitalWrite(PIN_CMD_TRANSISTOR, HIGH);

  delay(200);

  DS3231_init(DS3231_CONTROL_INTCN);
    DS3231_clear_a1f();
    DS3231_clear_a2f();

  // following line sets the RTC to the date & time this sketch was compiled
  //   rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  //   rtc.adjust(DateTime(2020, 1, 13, 16, 20, 0));  // Défini une date et une heure pour le RTC (à ne faire qu'une fois)
  // Par exemple pour le 9 avril 2018 à 16:36:20
  // rtc.adjust(DateTime(2018, 4 ,9 , 16 ,36 ,20));

   if (!ENV.begin()) {
    Serial.println("Failed to initialize MKR ENV Shield!");  // Bloque le programme si probleme
    while (1){
      digitalWrite(PIN_LED_BUILTIN, HIGH);
      delay(500);
      digitalWrite(PIN_LED_BUILTIN, LOW);
      delay(500);
    }
  }
  digitalWrite(PIN_LED_BUILTIN, HIGH);

  if (!SD.begin(SDCARD_CS))  // Si on ne détecte pas la SD
  {
    while (1) {
      digitalWrite(PIN_LED_BUILTIN, HIGH);  // Bloque le programme si probleme
      delay(200);
      digitalWrite(PIN_LED_BUILTIN, LOW);
      delay(300);
    }
  }
  digitalWrite(PIN_LED_BUILTIN, HIGH);
  
  set_next_alarm();

  if (SD.exists("datalog.txt")) {
    File dataFile = SD.open("datalog.txt", FILE_WRITE);
    dataFile.close();
  }
  else {
    // Ouvre/crée un fichier nommé "datalog" dans la carte SD
    File dataFile = SD.open("datalog.txt", FILE_WRITE);  
    // Ecriture dans le fichier de l'entête des données
    dataFile.println("Date, Heure, Temperature, Humidity, Pressure, Illuminance");
    // Ferme le fichier de la carte SD
    dataFile.close();  
  }
}

void loop() {

  delay(10000);
  
  Dtime = rtc.now();  // Défini la date actuelle comme celle du RTC
  // Dtime = DateTime(y, m, d, hh, mm, ss) on veut day/month/year, hh:mm:ss,
  String Sdate = String(Dtime.day()) + "/" + String(Dtime.month()) + "/" + String(Dtime.year());
  String Stime = String(Dtime.hour())+ ":" + String(Dtime.minute()) + ":" + String(Dtime.second());
  
  // read all the sensor values
  float temperature = ENV.readTemperature();
  float humidity = ENV.readHumidity();
  float pressure = ENV.readPressure();
  float illuminance = ENV.readIlluminance();
  /* float uva = ENV.readUVA();
  float uvb = ENV.readUVB();
  float uvIndex = ENV.readUVIndex(); */ 

  File fdata = SD.open("datalog.txt", FILE_WRITE);  // Ouvre/crée un fichier nommé "datalog" dans la carte SD

  fdata.println(Sdate + ", " + Stime + ", " + \
                String(temperature, 2) + ", " + \
                String(humidity, 2) + ", " + \
                String(pressure, 2) + ", " + \
                String(illuminance, 2));

  fdata.close();
                       
  while (1) {
    digitalWrite(PIN_CMD_TRANSISTOR, LOW);                                                                                   //Sortie pin 7 LOW  POUR MISE EN SOMMEIL DE LA CARTE MKR
    delay(5000);  // attente bloquante de 5 sec
  }
}
